#!/bin/sh
#实时流量监控
#请加入定时任务

NAME_TAG=("OpenVPN CLIENT LIST" "ROUTING TABLE" "GLOBAL STATS" "END")
LOG_FILE="/var/www/html/res/openvpn-status.txt"

DB='ov'
DBADMIN='root'
DBHOST='localhost'
DBPASSWD='MySQLPass'

flow_info_index=0
function IsFlowInfoStatus()
{
    if [ "${1}" == "${NAME_TAG[0]}" ]
    then
        #进入客户端列表，返回[1]
        flow_info_index=1
        return 1
    elif [ "${1}" == "${NAME_TAG[1]}" ] || [ "${1}" == "${NAME_TAG[2]}" ] || [ "${1}" == "${NAME_TAG[3]}" ]
    then
        #进入其他信息，返回结束[0]
        flow_info_index=0
        return 0
    fi

    if [ ${flow_info_index} -lt 1 ]
    then
        #没有进入客户端列表，返回[0]
        return 0
    fi
    
    let "flow_info_index+=1"
    if [ ${flow_info_index} -lt 4 ]
    then
        #进入客户端列表，还没有进入IP列表，返回[1]
        return 1
    fi
    
    
    #进入到IP列表，返回[2]
    return 2;
}

flow_infos=("")
function GetFlowInfo()
{
    OLD_IFS=${IFS}
    IFS=$2
    unset flow_infos
    flow_infos=(${1})
    IFS=${OLD_IFS}
    
    #echo "flow_infos: ${#flow_infos[@]}"
}


function StringSplit()
{
    local OLD_IFS=${IFS}
    IFS=$2
    local array=(${1})
    IFS=${OLD_IFS}
    #echo ${array[@]}
}


function UpdateUserFlowToDB()
{
    local common_name=${flow_infos[0]}
    local bytes_received=${flow_infos[2]}
    local bytes_sent=${flow_infos[3]}
    #echo "username: $common_name received: $bytes_received sents: $bytes_sent ip_pro:${flow_infos[1]}"
    
    local SQL="SELECT isent FROM openvpn WHERE iuser='$common_name';"
    local isenta=($(mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "$SQL" $DB))

    local SQL="SELECT irecv FROM openvpn WHERE iuser='$common_name';"
    local irecva=($(mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "$SQL" $DB))

    recv=$[${irecva[1]}+$bytes_sent]
    sent=$[${isenta[1]}+$bytes_received]
    all=$[$recv+$sent]
    #echo "username: $common_name recv: $recv sent: $sent all:$all"
    #更新日志、统计流量
    #mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET isent=$sent,irecv=$recv WHERE iuser='$common_name';" $DB

    #禁用到期账号
    #mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET i=0 WHERE iuser='$common_name' AND UNIX_TIMESTAMP(NOW())>endtime;" $DB

    #查询用户是否超量
    local SQL="SELECT count(*) FROM openvpn WHERE $all>=maxll AND iuser='$common_name';"
    local COUNT=($(mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "$SQL" $DB))
    if [ ${COUNT[1]} -ne 0 ]
    then
       #禁用超量用户
       mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET i=0 WHERE iuser='$common_name';" $DB
       #踢出超量用户 采用用户名
       bash /etc/openvpn/kill.sh ${flow_infos[0]}
       #更新超量用户流量
       mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET i=1,isent=$sent,irecv=$recv WHERE iuser='$common_name';" $DB
       #更新日志
       mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "INSERT INTO auth_log(action,msg,time) VALUES ('踢出超量','用户名: "${flow_infos[0]}"  ip_端口:"${flow_infos[1]}"',NOW());" $DB
    fi

    #查询用户是否过期
    local SQL="SELECT count(*) FROM openvpn WHERE UNIX_TIMESTAMP(NOW())>endtime AND iuser='$common_name';"
    local COUNT=($(mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "$SQL" $DB))
    if [ ${COUNT[1]} -ne 0 ]
    then
       #禁用过期用户
       mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET i=0 WHERE iuser='$common_name';" $DB
       #踢出过期用户 采用用户名
       bash /etc/openvpn/kill.sh ${flow_infos[0]}
       #清零过期用户流量
       mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET i=1,isent=0,irecv=0,maxll=0 WHERE iuser='$common_name';" $DB
       #更新日志
       mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "INSERT INTO auth_log(action,msg,time) VALUES ('踢出过期','用户名: "${flow_infos[0]}"  ip_端口:"${flow_infos[1]}"',NOW());" $DB
    fi
}

while read LOG_FILE; do
    
    IsFlowInfoStatus "${LOG_FILE}"
    status=$?
    #echo "output: ${status}"
    if [ ${status} -eq 2 ]
    then
        
        GetFlowInfo "${LOG_FILE}" ","
        if [ ${#flow_infos[@]} -ne 5 ]
        then
            continue
        fi
        
        UpdateUserFlowToDB
    fi
    
done < ${LOG_FILE}

#清零所有到期账号流量
mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET isent=0,irecv=0,maxll=0 WHERE UNIX_TIMESTAMP(NOW())>endtime;" $DB
#清零所有超量账号流量
mysql -h$DBHOST -u$DBADMIN -p$DBPASSWD -e "UPDATE openvpn SET irecv=maxll-isent WHERE isent+irecv>maxll;" $DB